#-*- coding:utf-8 -*-
'''查找当前目录或者指定目录下所有har文件，并调用har2case批量转换为py文件'''

from genericpath import isfile
import sys, os, jinja2
# import subprocess
# from subprocess import TimeoutExpired, Popen, PIPE
from tslrunner import utils
import argparse
import shutil

root_path = None

__TEMPLATE__ = jinja2.Template(
"""# NOTE: Generated By tslRunner
# FROM: {{ root_path }}
import os, sys, pytest
from pathlib import Path

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

from tslrunner import Parameters
from tslrunner import HttpRunner, Config, Step, RunRequest, RunTestCase
{% for import_str in imports_list %}
{{ import_str }}
{% endfor %}

@pytest.mark.normal
class TestSuite(HttpRunner):
    #@pytest.mark.parametrize("param", Parameters({'param1-param2-param3': '${P(data/test.csv)}'}),)
    #def test_start(self, param):
    #    super().test_start(param)

    config = Config("root_path").verify(False)

    teststeps = [
        {% for case in case_list %}
        Step(
            {{ case }}
        ),
        {% endfor %}
    ]

if __name__ == "__main__":
    TestSuite().test_start()

"""
)

def listdir(path, list_name): 
  for file in os.listdir(path): 
    file_path = os.path.join(path, file) 
    if os.path.isdir(file_path): 
      listdir(file_path, list_name) 
    elif os.path.splitext(file_path)[1]=='.har': 
      list_name.append(file_path)

# def all_decode(content):
#     result = None
#     for _charset_ in ['utf-8', 'gbk', 'gb2312']:
#         if content:
#             result = content.decode(_charset_, errors='replace')
#             if '�' not in result.strip():
#                 result = content.decode(_charset_, errors='replace')
#                 break
#     if not result:
#     # 默认 utf-8
#         result = content.decode(_charset_, errors='replace')

#     return result


# def cmd(command):
#     proc = subprocess.Popen(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
#     try:
#         errs, outs = proc.communicate(timeout=15)
#         print(all_decode(outs))
#         print(all_decode(errs))
#         # print(outs.decode("gbk"))
#         #不转码有乱码问题。转码又不能兼容所有系统
#         # print(errs.decode("gbk"))
#     except TimeoutExpired:
#         proc.kill()
#         outs, errs = proc.communicate()
    
def add_env_and_data_file():
    currentPath = os.path.dirname(os.path.abspath(__file__))
    path = os.path.dirname(os.path.realpath(__file__)) + "/demo/"
    if not os.path.exists(".env"):
        shutil.copy(path + "env", os.getcwd() + "/.env")
    
    if not os.path.exists("requirements.txt"):
        shutil.copy(path + "requirements.txt", os.getcwd())
    if not os.path.exists("pytest.ini"):
        shutil.copy(path+"pytest.ini", os.getcwd())
    if not os.path.exists("data"):
        shutil.copytree(path + "/data", os.getcwd() + "/data")
    
def generate_suite_file(path, fileList):
    
    modulePath = os.path.split(path)[1]
    imports_list = []
    case_list = []
    tmpName = ""
    loopNum = 0
    for file in fileList:
        if loopNum == 0:
            tmpName += file + "_test"
        else:
            tmpName += " ," + file + "_test"
        loopNum += 1
        
    import_expr = f"from {modulePath} import {tmpName}"
    if import_expr not in imports_list:
        imports_list.append(import_expr)

    if isinstance(fileList,list):
        for file in fileList:
            case_list.append(f"RunTestCase(\"{file}\").call({file}_test.TestCase{file})")
    else:
        case_list.append(f"RunTestCase(\"{fileList}\").call({fileList}_test.TestCase{fileList})")

    data = {
            "testcase_path": path,
            "imports_list": imports_list,
            "case_list": case_list,
            "root_path": root_path,
    }

    content = __TEMPLATE__.render(data)
    # print(content)
    with open("Test" + modulePath + ".py", "w", encoding="utf-8") as f:
        f.write(content)

def main():
    # print(os.getcwd())
    # print(os.path.abspath('.'))
    list_name = []

    parser = argparse.ArgumentParser()
    parser.add_argument("har2case_batch", help="batch Convert HAR(HTTP Archive) to pytest case.")
    parser.add_argument("har_source_file", nargs="?", help="Specify HAR source file")
    parser.add_argument(
        "-add",
        dest="add_env_file",
        action="store_true",
        help="add .env and /data/test.csv file"
    )

    parser.add_argument(
        "-gen",
        "--gen_suite",
        dest="gen_suite_file",
        action="store_true",
        help="generate suite file from every folder file"
    )

    parser_args, unknownargs = parser.parse_known_args()

    unknownargs = parser.parse_known_args()

    har_path = parser_args.har_source_file


    if parser_args.add_env_file:
        add_env_and_data_file()

    root_path = None
    pathMap = {}
    if har_path == None:
        # tslrunner har2case batch
        path = os.path.abspath('.')
        root_path = path
        listdir(path, list_name)
        # print(list_name)
        
    elif har_path:
        
        path = har_path
        if os.path.isdir(path):
            root_path = path
            listdir(path, list_name)
        elif os.path.isfile(path):
            root_path = os.path.dirname(path)
            list_name.append(path)
        # print(list_name)
    
    baseCmd = "har2case "
    command = ""

    extArgs = unknownargs[1]
    ext = ""
    if extArgs:
        for arg in extArgs:
            ext = ext + " " + arg
    # print(args)
    for file in list_name:
        path = os.path.dirname(file)
        fileName, _ = os.path.splitext(os.path.basename(file))
        os.chdir(path)
        # fileName = os.path.split(file)[1]
        command = baseCmd + file + ext
        print(command)
        utils.cmd(command)

        if root_path != path:
            if path in pathMap:
                pathMap[path].append(fileName)
            else:
                pathMap[path] = [fileName]

    print(pathMap)
    #生成文件完成后记得返回当前项目根目录
    os.chdir(root_path)
    for path in pathMap.keys():
        if parser_args.gen_suite_file:
            generate_suite_file(path, pathMap[path])
    
    
    sys.exit(0)

if __name__ == "__main__":
    main()