#-*- coding:utf-8 -*-
'''查找当前目录或者指定目录下所有har文件，并调用har2case批量转换为py文件'''

from cProfile import run
import sys, os, jinja2
import argparse
import shutil, re
from os import environ
from tslrunner import loader, exceptions, utils

root_path = None
tag = "normal"
env_param = None
project_name = None
app_name = None
run_file = "run_all_test.py"
run_temp_file = "run_all_test.py.tmp"
env_file = ".env"
conftest_file = "conftest.py"


__TEMPLATE__ = jinja2.Template(
"""# NOTE: Generated By tslRunner
import pytest
import utils
import sys, os
{% for folder in imports_list %}
from {{folder}} import *
{% endfor %}

if __name__ == '__main__':
    pytest.main([
    "-s",
    {# 测试suite名，每行一个增加在此处，运行的时候全部运行 #}
    {% for file in file_list %}
    "{{file}}",
    {% endfor %}
    "-m {{tag}}",
    "--html=report.html", "--self-contained-html", 
    "--capture=sys", "--json-report", "--json=report.json", "--json-report-omit=collectors log traceback streams warnings keywords",
    #  "--junitxml=result.xml",
     ])

    utils.get_magicbow_and_send(sys.argv)

    utils.cp_result_to_magicbow()

"""
)

def listdir(path, list_name, sub_folder): 
  for file in os.listdir(path):
    file_path = os.path.join(path, file) 
    #查找suite，只找顶层目录，不找下级目录
    if os.path.isdir(file_path):
        folder_basename = os.path.basename(file_path)
        #print("check folder: ", folder_basename)
        if "data" != folder_basename and not folder_basename.startswith(".") \
            and not folder_basename.startswith("_"):
            sub_folder.append(folder_basename.replace(".\\", ""))
    if os.path.isfile(file_path):
        #print("check file: ", file_path)
        file_name = os.path.basename(file_path)
        file_name_arg = os.path.splitext(file_name)
        if  file_name_arg[0].startswith("Test") and file_name_arg[1]=='.py': 
            list_name.append(file_name)
            
def add_conf_and_utils_file(path):
    os.chdir(os.path.abspath(path))
    currentPath = os.path.dirname(os.path.abspath(__file__))
    path = os.path.dirname(os.path.realpath(__file__)) + "/demo/"

    if not os.path.exists("debugtalk.py"):
        shutil.copy(path+"debugtalk.py", os.getcwd())

    if not os.path.exists("conftest.py"):
        shutil.copy(path+"conftest.py", os.getcwd())
    
    if not os.path.exists("utils.py"):
        shutil.copy(path+"utils.py", os.getcwd())
    

def generate_run_file(path, fileList, sub_folder):

    data = {
            "testcase_path": path,
            "imports_list": sub_folder,
            "file_list": fileList,
            "root_path": root_path,
            "tag": tag
    }

    content = __TEMPLATE__.render(data)
    # print(content)
    #如果文件存在，不替换，但是需要替换tag标签
    if not os.path.exists(run_file):
        with open(run_file, "w", encoding="utf-8") as f:
            f.write(content)
    else:
        file = open(run_file,"r+", encoding="utf-8")
        with open(run_temp_file, 'w', encoding='UTF-8') as tmp:
            tmp.write(re.sub(r'-m\s+\w*', '-m ' + tag, file.read()))
        file.close()
        if os.path.exists(run_file):
            os.remove(run_file)
        
        shutil.copy(run_temp_file, run_file)
        os.remove(run_temp_file)

def write_mapping_to_env(mapping):
    with open(env_file, "w", encoding="UTF-8") as fp:
            for key, value in mapping.items():
                fp.write(key + "=" + value)
                fp.write('\n')

def replace_env(env_param):
    global env_file
    param_list = None
    param_mapping = {}
    if env_param:
        param_list = str(env_param).split(";")
        for line in param_list:
            if "=" in line:
                variable, value = line.split("=", 1)
            elif ":" in line:
                variable, value = line.split(":", 1)
            else:
                raise exceptions.EnvNotFound("-env param format error")

            param_mapping[variable.strip()] = value.strip()
        print("replace env param: ", param_mapping)

    if os.path.exists(env_file):
        #读取env文件内容
        env_variables_mapping = loader.load_dot_env_file(env_file)
        print("替换env参数: ", env_variables_mapping)
        for key, value in param_mapping.items():
            env_variables_mapping[key] = value

    write_mapping_to_env(param_mapping)

def main():
    global root_path
    list_name = []
    sub_folder = []

    parser = argparse.ArgumentParser()
    parser.add_argument("tslrun", help="tslrunner run suite command.")
    parser.add_argument("suite_folder", nargs="?", help="test suite source file")
    parser.add_argument(
        "-tag",
        dest="tag_mark",
        help="only run tag marked suite"
    )
    parser.add_argument(
        "-env",
        dest="env_replace",
        help="replace env param in .env file"
    )
    parser.add_argument(
        "-project",
        dest="project_name",
        help="replace project name in conftest file"
    )
    parser.add_argument(
        "-app",
        dest="app_name",
        help="replace app name in conftest file"
    )
    
    parser_args, unknownargs = parser.parse_known_args()

    unknownargs = parser.parse_known_args()

    suite_folder = parser_args.suite_folder

    #遍历工作目录下所有py文件和子目录用于组装run_all_test.py文件
    if suite_folder == None:
        # tslrunner project folder find suite 
        path = os.path.abspath('.')
        root_path = path
        listdir(path, list_name, sub_folder)
        print(".工作目录suite文件列表： ", list_name)
        print(".用例子目录列表： ", sub_folder)
        
    elif suite_folder:
        path = suite_folder
        if os.path.isdir(path):
            os.chdir(path)
            root_path = os.path.realpath(os.getcwd())
            listdir(".", list_name, sub_folder)
        elif os.path.isfile(path):
            root_path = os.path.dirname(path)
            listdir(path, list_name, sub_folder)
        print("工作目录suite文件列表: ", list_name)
        print("用例子目录列表: ", sub_folder)
    
    #复制配置和工具文件到工作目录
    add_conf_and_utils_file(root_path)

    if parser_args.tag_mark:
        global tag 
        tag = parser_args.tag_mark

    if parser_args.env_replace:
        global env_param
        env_param = parser_args.env_replace
        replace_env(env_param)

    if parser_args.project_name:
        global project_name
        project_name = parser_args.project_name
        utils.replace_file_content(conftest_file, "config._metadata[\"项目名称\"] =", " \"" + project_name + "\"")
    
    if parser_args.app_name:
        global app_name
        app_name = parser_args.app_name
        utils.replace_file_content(conftest_file, "config._metadata[\"应用名称\"] = ", " \"" + app_name + "\"")

    #替换执行人
    utils.replace_file_content(conftest_file, "prefix.append([html.th(\"执行人: \"),html.label(", "\"" + utils.get_builder_name() + "\")])")

    if len(list_name) > 0:
        generate_run_file(root_path, list_name, sub_folder)

    if os.path.exists(run_file):
        utils.cmd("python run_all_test.py")

    sys.exit(0)

if __name__ == "__main__":
    main()